@@ -1,9 +1,9 @@
 # SystemC - SystemC Perl Interface
-# $Id: Net.pm 8326 2005-11-02 19:13:56Z wsnyder $
+# $Id: Net.pm 11992 2006-01-16 18:59:58Z wsnyder $
 # Author: Wilson Snyder <wsnyder@wsnyder.org>
 ######################################################################
 #
-# Copyright 2001-2005 by Wilson Snyder.  This program is free software;
+# Copyright 2001-2006 by Wilson Snyder.  This program is free software;
 # you can redistribute it and/or modify it under the terms of either the GNU
 # General Public License or the Perl Artistic License.
 # 
@@ -20,7 +20,7 @@ use Class::Struct;
 use Verilog::Netlist;
 use SystemC::Netlist;
 @ISA = qw(Verilog::Netlist::Net);
-$VERSION = '1.240';
+$VERSION = '1.250';
 use strict;
 
 ######################################################################
@@ -118,8 +118,10 @@ sub verilog_text {
     my @out;
     foreach my $decl ($self->_scdecls) {
 	push @out, $decl;
-	push @out, " [".$self->msb.":".$self->lsb."]"
-	    if defined $self->msb && !($self->msb==0 && $self->lsb==0);
+	my $vec = ((defined $self->msb && !($self->msb==0 && $self->lsb==0))
+		   ? " [".$self->msb.":".$self->lsb."]"
+		   : "");
+	push @out, sprintf("%-8s", $vec);
 	push @out, " ".$self->name;
 	push @out, " ".$self->array if $self->array;
 	push @out, ";";
@@ -147,7 +149,7 @@ pin.
 
 The latest version is available from CPAN and from L<http://www.veripool.com/>.
 
-Copyright 2001-2005 by Wilson Snyder.  This package is free software; you
+Copyright 2001-2006 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
 Lesser General Public License or the Perl Artistic License.
 
